
; damage_mod_fe7.asm
; FE7 Damage modifier written by Glain

; This is my first GBA hack. Testing has been fairly sparse and only on the VBA debug emulator, so use at your own risk!

.gba
.open "fe7_mod.gba",0x08000000

.thumb

.org 0x08D00100

; Roll function: returns integer RN between (r0 = Lowest) and (r1 = Highest), inclusive
func_roll:

push  {r7,lr}
mov   r7, sp

sub   r2, r1, r0
add   r2, r2, #0x1		; r2 = Number of possible values from roll
push  {r0}
push  {r2}

; RNG call: $08000be0
mov   r2, #0x08
lsl   r2, #20
add   r2, #0xbe
lsl   r2, #4
add   r2, #1
mov   r0, pc
add   r0, #5
mov   lr, r0
bx    r2	                  ; r0 = RN (Result of RNG call) 

pop   {r2}
add   r1, r0, #0x0
mul   r1, r2
add   r0, r1, #0x0
cmp   r0, #0x0
bge   func_roll_end

mov   r1, #0xff			; r1 = 0x0000ffff
lsl   r1, #8
add   r1, #0xff
add   r0, r0, r1

func_roll_end:
asr   r1, r0, #0x10
pop   {r0}
add   r0, r0, r1

pop   {r7}
pop   {r1}
bx    r1

.org 0x08D00200

; Damage modifier function: 
;     Parameters: r0 = damage (pre-modified); r1 = Low multiplier; r2 = High multiplier (Multipliers out of 128)
;     Result: r0 = New damage
func_damage_mod:

push  {r7,lr}
mov   r7, sp

; Generate RN between low and high (inclusive)
push  {r0}
add   r0, r1, #0
add   r1, r2, #0
bl    func_roll
pop   {r1}

; Damage = (pre_damage * (low..high) + 64) / 128
mul   r1, r0
add   r0, r1, #0
add   r0, #64
asr   r0, r0, #7

pop   {r7}
pop   {r1}
bx    r1

.org 0x08D00250

; Normal damage modifier function
func_damage_mod_normal:

.arm

mov   r2, pc
add   r2, #5
bx    r2                ; Switch to THUMB mode

.thumb

push  {r7,lr}
mov   r7, sp

ldrh  r1, [r4, #0x6] 	; Load attack
ldrh  r2, [r4, #0x8]	; Load defense
sub   r0, r1, r2		; Damage = attack - defense

push  {r0,r1,r2,r3}
mov   r1, #96
mov   r2, #160
bl    func_damage_mod
strh  r0, [r4, #0x4]	
pop   {r0,r1,r2,r3}

pop   {r7}
pop   {r1}
add   r1, 1             ; Back to THUMB mode
bx    r1

.org 0x08D002A0

; Normal critical damage modifier function
func_damage_mod_critical:

.arm

mov   r2, pc
add   r2, #5
bx    r2                ; Switch to THUMB mode

.thumb

push  {r7,lr}
mov   r7, sp

mov   r1, #0x4		
ldsh  r0, [r4, r1]      ; Load damage

push  {r0,r1,r2,r3}
mov   r1, #255          ; r1 = 256
add   r1, #1
mov   r2, #255          ; r2 = 307
add   r2, #57
bl    func_damage_mod
strh  r0, [r4, #0x4]	
pop   {r0,r1,r2,r3}

pop   {r7}
pop   {r1}
add   r1, 1             ; Back to THUMB mode
bx    r1

.org 0x08029294

.thumb
nop
bx    pc                ; Switch to ARM mode
.arm
bl    func_damage_mod_normal

.org 0x080292E0 

.thumb
nop
bx    pc                ; Switch to ARM mode
.arm
bl    func_damage_mod_critical
.thumb
nop

.close

; End of ASM file (damage_mod_fe7.asm)
